import 'dart:async';
import 'dart:io';
import 'dart:convert';
import 'package:dio/dio.dart';
import 'package:built_collection/built_collection.dart';
import 'package:built_value/serializer.dart';

import 'package:komga_api_client/model/collection_dto.dart';
import 'package:komga_api_client/model/collection_creation_dto.dart';
import 'package:komga_api_client/model/page_series_dto.dart';
import 'package:komga_api_client/model/page_collection_dto.dart';
import 'package:komga_api_client/model/collection_update_dto.dart';
import 'dart:typed_data';

class SeriesCollectionControllerApi {
    final Dio _dio;
    Serializers _serializers;

    SeriesCollectionControllerApi(this._dio, this._serializers);

        /// 
        ///
        /// 
        Future<Response<CollectionDto>>addOne1(CollectionCreationDto collectionCreationDto,{ CancelToken cancelToken, Map<String, String> headers,}) async {

        String _path = "/api/v1/collections";

        Map<String, dynamic> queryParams = {};
        Map<String, String> headerParams = Map.from(headers ?? {});
        dynamic bodyData;

        queryParams.removeWhere((key, value) => value == null);
        headerParams.removeWhere((key, value) => value == null);

        List<String> contentTypes = ["application/json"];


            var serializedBody = _serializers.serialize(collectionCreationDto);
            var jsoncollectionCreationDto = json.encode(serializedBody);
            bodyData = jsoncollectionCreationDto;

            return _dio.request(
            _path,
            queryParameters: queryParams,
            data: bodyData,
            options: Options(
            method: 'post'.toUpperCase(),
            headers: headerParams,
            extra: {
                'secure': [],
            },
            contentType: contentTypes.isNotEmpty ? contentTypes[0] : "application/json",
            ),
            cancelToken: cancelToken,
            ).then((response) {

        var serializer = _serializers.serializerForType(CollectionDto);
        var data = _serializers.deserializeWith<CollectionDto>(serializer, response.data is String ? jsonDecode(response.data) : response.data);

            return Response<CollectionDto>(
                data: data,
                headers: response.headers,
                request: response.request,
                redirects: response.redirects,
                statusCode: response.statusCode,
                statusMessage: response.statusMessage,
                extra: response.extra,
            );
            });
            }
        /// 
        ///
        /// 
        Future<Response>deleteOne1(int id,{ CancelToken cancelToken, Map<String, String> headers,}) async {

        String _path = "/api/v1/collections/{id}".replaceAll("{" r'id' "}", id.toString());

        Map<String, dynamic> queryParams = {};
        Map<String, String> headerParams = Map.from(headers ?? {});
        dynamic bodyData;

        queryParams.removeWhere((key, value) => value == null);
        headerParams.removeWhere((key, value) => value == null);

        List<String> contentTypes = [];



            return _dio.request(
            _path,
            queryParameters: queryParams,
            data: bodyData,
            options: Options(
            method: 'delete'.toUpperCase(),
            headers: headerParams,
            extra: {
                'secure': [],
            },
            contentType: contentTypes.isNotEmpty ? contentTypes[0] : "application/json",
            ),
            cancelToken: cancelToken,
            );
            }
        /// 
        ///
        /// 
        Future<Response<PageCollectionDto>>getAll1({ String search,List<int> libraryId,bool unpaged,int page,int size,CancelToken cancelToken, Map<String, String> headers,}) async {

        String _path = "/api/v1/collections";

        Map<String, dynamic> queryParams = {};
        Map<String, String> headerParams = Map.from(headers ?? {});
        dynamic bodyData;

                queryParams[r'search'] = search;
                queryParams[r'library_id'] = libraryId;
                queryParams[r'unpaged'] = unpaged;
                queryParams[r'page'] = page;
                queryParams[r'size'] = size;
        queryParams.removeWhere((key, value) => value == null);
        headerParams.removeWhere((key, value) => value == null);

        List<String> contentTypes = [];



            return _dio.request(
            _path,
            queryParameters: queryParams,
            data: bodyData,
            options: Options(
            method: 'get'.toUpperCase(),
            headers: headerParams,
            extra: {
                'secure': [],
            },
            contentType: contentTypes.isNotEmpty ? contentTypes[0] : "application/json",
            ),
            cancelToken: cancelToken,
            ).then((response) {

        var serializer = _serializers.serializerForType(PageCollectionDto);
        var data = _serializers.deserializeWith<PageCollectionDto>(serializer, response.data is String ? jsonDecode(response.data) : response.data);

            return Response<PageCollectionDto>(
                data: data,
                headers: response.headers,
                request: response.request,
                redirects: response.redirects,
                statusCode: response.statusCode,
                statusMessage: response.statusMessage,
                extra: response.extra,
            );
            });
            }
        /// 
        ///
        /// 
        Future<Response<Uint8List>>getCollectionThumbnail(int id,{ CancelToken cancelToken, Map<String, String> headers,}) async {

        String _path = "/api/v1/collections/{id}/thumbnail".replaceAll("{" r'id' "}", id.toString());

        Map<String, dynamic> queryParams = {};
        Map<String, String> headerParams = Map.from(headers ?? {});
        dynamic bodyData;

        queryParams.removeWhere((key, value) => value == null);
        headerParams.removeWhere((key, value) => value == null);

        List<String> contentTypes = [];



            return _dio.request(
            _path,
            queryParameters: queryParams,
            data: bodyData,
            options: Options(
            method: 'get'.toUpperCase(),
            headers: headerParams,
            extra: {
                'secure': [],
            },
            contentType: contentTypes.isNotEmpty ? contentTypes[0] : "application/json",
            ),
            cancelToken: cancelToken,
            ).then((response) {

        var serializer = _serializers.serializerForType(Uint8List);
        var data = _serializers.deserializeWith<Uint8List>(serializer, response.data is String ? jsonDecode(response.data) : response.data);

            return Response<Uint8List>(
                data: data,
                headers: response.headers,
                request: response.request,
                redirects: response.redirects,
                statusCode: response.statusCode,
                statusMessage: response.statusMessage,
                extra: response.extra,
            );
            });
            }
        /// 
        ///
        /// 
        Future<Response<CollectionDto>>getOne1(int id,{ CancelToken cancelToken, Map<String, String> headers,}) async {

        String _path = "/api/v1/collections/{id}".replaceAll("{" r'id' "}", id.toString());

        Map<String, dynamic> queryParams = {};
        Map<String, String> headerParams = Map.from(headers ?? {});
        dynamic bodyData;

        queryParams.removeWhere((key, value) => value == null);
        headerParams.removeWhere((key, value) => value == null);

        List<String> contentTypes = [];



            return _dio.request(
            _path,
            queryParameters: queryParams,
            data: bodyData,
            options: Options(
            method: 'get'.toUpperCase(),
            headers: headerParams,
            extra: {
                'secure': [],
            },
            contentType: contentTypes.isNotEmpty ? contentTypes[0] : "application/json",
            ),
            cancelToken: cancelToken,
            ).then((response) {

        var serializer = _serializers.serializerForType(CollectionDto);
        var data = _serializers.deserializeWith<CollectionDto>(serializer, response.data is String ? jsonDecode(response.data) : response.data);

            return Response<CollectionDto>(
                data: data,
                headers: response.headers,
                request: response.request,
                redirects: response.redirects,
                statusCode: response.statusCode,
                statusMessage: response.statusMessage,
                extra: response.extra,
            );
            });
            }
        /// 
        ///
        /// 
        Future<Response<PageSeriesDto>>getSeriesForCollection(int id,{ bool unpaged,int page,int size,CancelToken cancelToken, Map<String, String> headers,}) async {

        String _path = "/api/v1/collections/{id}/series".replaceAll("{" r'id' "}", id.toString());

        Map<String, dynamic> queryParams = {};
        Map<String, String> headerParams = Map.from(headers ?? {});
        dynamic bodyData;

                queryParams[r'unpaged'] = unpaged;
                queryParams[r'page'] = page;
                queryParams[r'size'] = size;
        queryParams.removeWhere((key, value) => value == null);
        headerParams.removeWhere((key, value) => value == null);

        List<String> contentTypes = [];



            return _dio.request(
            _path,
            queryParameters: queryParams,
            data: bodyData,
            options: Options(
            method: 'get'.toUpperCase(),
            headers: headerParams,
            extra: {
                'secure': [],
            },
            contentType: contentTypes.isNotEmpty ? contentTypes[0] : "application/json",
            ),
            cancelToken: cancelToken,
            ).then((response) {

        var serializer = _serializers.serializerForType(PageSeriesDto);
        var data = _serializers.deserializeWith<PageSeriesDto>(serializer, response.data is String ? jsonDecode(response.data) : response.data);

            return Response<PageSeriesDto>(
                data: data,
                headers: response.headers,
                request: response.request,
                redirects: response.redirects,
                statusCode: response.statusCode,
                statusMessage: response.statusMessage,
                extra: response.extra,
            );
            });
            }
        /// 
        ///
        /// 
        Future<Response>updateOne(int id,CollectionUpdateDto collectionUpdateDto,{ CancelToken cancelToken, Map<String, String> headers,}) async {

        String _path = "/api/v1/collections/{id}".replaceAll("{" r'id' "}", id.toString());

        Map<String, dynamic> queryParams = {};
        Map<String, String> headerParams = Map.from(headers ?? {});
        dynamic bodyData;

        queryParams.removeWhere((key, value) => value == null);
        headerParams.removeWhere((key, value) => value == null);

        List<String> contentTypes = ["application/json"];


            var serializedBody = _serializers.serialize(collectionUpdateDto);
            var jsoncollectionUpdateDto = json.encode(serializedBody);
            bodyData = jsoncollectionUpdateDto;

            return _dio.request(
            _path,
            queryParameters: queryParams,
            data: bodyData,
            options: Options(
            method: 'patch'.toUpperCase(),
            headers: headerParams,
            extra: {
                'secure': [],
            },
            contentType: contentTypes.isNotEmpty ? contentTypes[0] : "application/json",
            ),
            cancelToken: cancelToken,
            );
            }
        }
